@@ -1,4 +1,3 @@
-/* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* chatty-chat-view.h
  *
  * Copyright 2020 Purism SPC
@@ -13,6 +12,7 @@
 
 #include <gtk/gtk.h>
 
+#include "chatty-history.h"
 #include "chatty-chat.h"
 
 G_BEGIN_DECLS
@@ -25,5 +25,7 @@ GtkWidget  *chatty_chat_view_new      (void);
 void        chatty_chat_view_set_chat (ChattyChatView *self,
                                        ChattyChat     *chat);
 ChattyChat *chatty_chat_view_get_chat (ChattyChatView *self);
+void        chatty_chat_view_set_db   (ChattyChatView *self,
+                                       ChattyHistory  *history);
 
 G_END_DECLS
