@@ -1,4 +1,3 @@
-/* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* chatty-list-row.c
  *
  * Copyright 2020 Purism SPC
@@ -20,6 +19,7 @@
 #include "chatty-contact.h"
 #include "chatty-contact-list.h"
 #include "chatty-chat.h"
+#include "chatty-ma-key-chat.h"
 #include "chatty-avatar.h"
 #include "chatty-clock.h"
 #include "chatty-list-row.h"
@@ -29,6 +29,7 @@ struct _ChattyListRow
 {
   GtkListBoxRow  parent_instance;
 
+  GtkWidget     *content_grid;
   GtkWidget     *avatar;
   GtkWidget     *title;
   GtkWidget     *subtitle;
@@ -206,6 +207,17 @@ chatty_list_row_update (ChattyListRow *self)
     last_message_time = chatty_chat_get_last_msg_time (item);
     gtk_widget_set_visible (self->last_modified, last_message_time > 0);
 
+    if (chatty_chat_get_chat_state (CHATTY_CHAT (self->item)) == CHATTY_CHAT_INVITED ||
+        CHATTY_IS_MA_KEY_CHAT (self->item)) {
+      gtk_label_set_text (GTK_LABEL (self->unread_message_count), "⚫︎");
+      gtk_widget_show (self->unread_message_count);
+      gtk_container_child_set (GTK_CONTAINER (self->content_grid), self->unread_message_count,
+                               "top-attach", 0, NULL);
+    } else {
+      gtk_container_child_set (GTK_CONTAINER (self->content_grid), self->unread_message_count,
+                               "top-attach", 1, NULL);
+      }
+
     chatty_list_row_update_last_modified (self);
   }
 
@@ -294,6 +306,8 @@ chatty_list_row_class_init (ChattyListRowClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/sm/puri/Chatty/"
                                                "ui/chatty-list-row.ui");
+
+  gtk_widget_class_bind_template_child (widget_class, ChattyListRow, content_grid);
   gtk_widget_class_bind_template_child (widget_class, ChattyListRow, avatar);
   gtk_widget_class_bind_template_child (widget_class, ChattyListRow, checkbox);
   gtk_widget_class_bind_template_child (widget_class, ChattyListRow, close_button);
