@@ -1,4 +1,3 @@
-/* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* chatty-message.h
  *
  * Copyright 2020 Purism SPC
@@ -11,6 +10,8 @@
 
 #pragma once
 
+#define CMATRIX_USE_EXPERIMENTAL_API
+#include <cmatrix.h>
 #include <glib-object.h>
 
 #include "chatty-item.h"
@@ -29,6 +30,9 @@ ChattyMessage      *chatty_message_new             (ChattyItem         *user,
                                                     ChattyMsgType       type,
                                                     ChattyMsgDirection  direction,
                                                     ChattyMsgStatus     status);
+ChattyMessage      *chatty_message_new_from_event  (ChattyItem         *user,
+                                                    CmEvent            *event);
+CmEvent            *chatty_message_get_cm_event    (ChattyMessage      *self);
 
 const char         *chatty_message_get_subject     (ChattyMessage      *self);
 void                chatty_message_set_subject     (ChattyMessage      *self,
@@ -39,6 +43,15 @@ void                chatty_message_set_encrypted   (ChattyMessage      *self,
                                                     gboolean            is_encrypted);
 
 GList              *chatty_message_get_files       (ChattyMessage      *self);
+void                chatty_message_get_file_stream_async   (ChattyMessage       *self,
+                                                            ChattyFileInfo      *file,
+                                                            ChattyProtocol       protocol,
+                                                            GCancellable        *cancellable,
+                                                            GAsyncReadyCallback  callback,
+                                                            gpointer             user_data);
+GInputStream       *chatty_message_get_file_stream_finish  (ChattyMessage       *self,
+                                                            GAsyncResult        *result,
+                                                            GError             **error);
 void                chatty_message_set_files       (ChattyMessage      *self,
                                                     GList              *files);
 void                chatty_message_add_file_from_path (ChattyMessage   *self,
