@@ -1,4 +1,3 @@
-/* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* chatty-ma-account.h
  *
  * Copyright 2020 Purism SPC
@@ -12,6 +11,8 @@
 #pragma once
 
 #include <glib-object.h>
+#define CMATRIX_USE_EXPERIMENTAL_API
+#include "cmatrix.h"
 
 #include "chatty-chat.h"
 #include "chatty-enums.h"
@@ -23,22 +24,9 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (ChattyMaAccount, chatty_ma_account, CHATTY, MA_ACCOUNT, ChattyAccount)
 
-ChattyMaAccount  *chatty_ma_account_new                (const char      *username,
-                                                        const char      *password);
-gboolean         chatty_ma_account_can_connect         (ChattyMaAccount *self);
+ChattyMaAccount  *chatty_ma_account_new_from_client    (CmClient        *cm_client);
+CmClient         *chatty_ma_account_get_cm_client      (ChattyMaAccount *self);
 const char       *chatty_ma_account_get_login_username (ChattyMaAccount *self);
-ChattyMaAccount  *chatty_ma_account_new_secret         (gpointer         secret_retrievable);
-void              chatty_ma_account_set_db             (ChattyMaAccount *self,
-                                                        gpointer         matrix_db,
-                                                        gpointer         history_db);
-void              chatty_ma_account_save_async         (ChattyMaAccount *self,
-                                                        gboolean         force,
-                                                        GCancellable    *cancellable,
-                                                        GAsyncReadyCallback callback,
-                                                        gpointer         user_data);
-gboolean          chatty_ma_account_save_finish        (ChattyMaAccount *self,
-                                                        GAsyncResult    *result,
-                                                        GError         **error);
 const char       *chatty_ma_account_get_homeserver     (ChattyMaAccount *self);
 void              chatty_ma_account_set_homeserver     (ChattyMaAccount *self,
                                                         const char      *server_url);
@@ -81,8 +69,4 @@ gboolean          chatty_ma_account_delete_3pid_finish (ChattyMaAccount *self,
                                                         GAsyncResult    *result,
                                                         GError         **error);
 
-/* For tests */
-void             chatty_ma_account_add_chat            (ChattyMaAccount *self,
-                                                        ChattyChat      *chat);
-
 G_END_DECLS
