@@ -1,4 +1,3 @@
-/* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* chatty-matrix.h
  *
  * Copyright 2021 Purism SPC
@@ -12,6 +11,8 @@
 #pragma once
 
 #include <glib-object.h>
+#define CMATRIX_USE_EXPERIMENTAL_API
+#include "cmatrix.h"
 
 #include "chatty-history.h"
 
@@ -21,9 +22,9 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (ChattyMatrix, chatty_matrix, CHATTY, MATRIX, GObject)
 
-ChattyMatrix  *chatty_matrix_new                     (ChattyHistory  *history,
-                                                      gboolean        disable_auto_login);
-void           chatty_matrix_load                    (ChattyMatrix   *self);
+ChattyMatrix   *chatty_matrix_new                     (gboolean        disable_auto_login);
+gboolean        chatty_matrix_is_enabled              (ChattyMatrix   *self);
+void            chatty_matrix_load                    (ChattyMatrix   *self);
 GListModel     *chatty_matrix_get_account_list       (ChattyMatrix   *self);
 GListModel     *chatty_matrix_get_chat_list          (ChattyMatrix   *self);
 void            chatty_matrix_delete_account_async   (ChattyMatrix   *self,
@@ -48,5 +49,8 @@ ChattyChat     *chatty_matrix_find_chat_with_name    (ChattyMatrix   *self,
                                                       ChattyProtocol  protocol,
                                                       const char     *account_id,
                                                       const char     *chat_id);
+CmClient       *chatty_matrix_client_new             (ChattyMatrix   *self);
+gboolean        chatty_matrix_has_user_id            (ChattyMatrix   *self,
+                                                      const char     *user_id);
 
 G_END_DECLS
